@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 #
 # mallocstacks  Trace malloc() calls in a process and print the full
 #               stack trace for all callsites.
@@ -25,7 +25,7 @@ b = BPF(text="""
 #include <uapi/linux/ptrace.h>
 
 BPF_HASH(calls, int);
-BPF_STACK_TRACE(stack_traces, 1024)
+BPF_STACK_TRACE(stack_traces, 1024);
 
 int alloc_enter(struct pt_regs *ctx, size_t size) {
     int key = stack_traces.get_stackid(ctx,
@@ -33,6 +33,7 @@ int alloc_enter(struct pt_regs *ctx, size_t size) {
     if (key < 0)
         return 0;
 
+    // could also use `calls.increment(key, size);`
     u64 zero = 0, *val;
     val = calls.lookup_or_init(&key, &zero);
     (*val) += size;
