@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 #
 # strlen_count  Trace strlen() and print a frequency count of strings.
 #               For Linux, uses BCC, eBPF. Embedded C.
@@ -31,6 +31,7 @@ int count(struct pt_regs *ctx) {
     u64 zero = 0, *val;
 
     bpf_probe_read(&key.c, sizeof(key.c), (void *)PT_REGS_PARM1(ctx));
+    // could also use `counts.increment(key)`
     val = counts.lookup_or_init(&key, &zero);
     (*val)++;
     return 0;
