@@ -104,6 +104,7 @@ class BPFTable : public BPFTableBase<void, void> {
   StatusTuple remove_value(const std::string& key_str);
 
   StatusTuple clear_table_non_atomic();
+  StatusTuple get_table_offline(std::vector<std::pair<std::string, std::string>> &res);
 
   static size_t get_possible_cpu_count();
 };
@@ -232,11 +233,14 @@ class BPFHashTable : public BPFTableBase<KeyType, ValueType> {
     KeyType cur;
     ValueType value;
 
+    StatusTuple r(0);
+
     if (!this->first(&cur))
       return res;
 
     while (true) {
-      if (!this->lookup(&cur, &value))
+      r = get_value(cur, value);
+      if (r.code() != 0)
         break;
       res.emplace_back(cur, value);
       if (!this->next(&cur, &cur))
@@ -364,4 +368,14 @@ class BPFCgroupArray : public BPFTableBase<int, int> {
   StatusTuple remove_value(const int& index);
 };
 
+class BPFDevmapTable : public BPFTableBase<int, int> {
+public:
+  BPFDevmapTable(const TableDesc& desc);
+  
+  StatusTuple update_value(const int& index, const int& value);
+  StatusTuple get_value(const int& index, int& value);
+  StatusTuple remove_value(const int& index);
+
+};
+
 }  // namespace ebpf
