@@ -18,6 +18,8 @@
 #include <string>
 #include <vector>
 #include <unistd.h>
+#include <errno.h>
+#include <ftw.h>
 
 namespace ebpf {
 
