@@ -48,6 +48,9 @@
 #include "libbpf.h"
 #include "perf_reader.h"
 
+// TODO: Remove this when CentOS 6 support is not needed anymore
+#include "setns.h"
+
 // TODO: remove these defines when linux-libc-dev exports them properly
 
 #ifndef __NR_bpf
@@ -74,9 +77,112 @@
 #define PERF_FLAG_FD_CLOEXEC (1UL << 3)
 #endif
 
+// TODO: Remove this when CentOS 6 support is not needed anymore
+#ifndef AF_ALG
+#define AF_ALG 38
+#endif
+
 #define min(x, y) ((x) < (y) ? (x) : (y))
 
-static int probe_perf_reader_page_cnt = 8;
+struct bpf_helper {
+  char *name;
+  char *required_version;
+};
+
+static struct bpf_helper helpers[] = {
+  {"map_lookup_elem", "3.19"},
+  {"map_update_elem", "3.19"},
+  {"map_delete_elem", "3.19"},
+  {"probe_read", "4.1"},
+  {"ktime_get_ns", "4.1"},
+  {"trace_printk", "4.1"},
+  {"get_prandom_u32", "4.1"},
+  {"get_smp_processor_id", "4.1"},
+  {"skb_store_bytes", "4.1"},
+  {"l3_csum_replace", "4.1"},
+  {"l4_csum_replace", "4.1"},
+  {"tail_call", "4.2"},
+  {"clone_redirect", "4.2"},
+  {"get_current_pid_tgid", "4.2"},
+  {"get_current_uid_gid", "4.2"},
+  {"get_current_comm", "4.2"},
+  {"get_cgroup_classid", "4.3"},
+  {"skb_vlan_push", "4.3"},
+  {"skb_vlan_pop", "4.3"},
+  {"skb_get_tunnel_key", "4.3"},
+  {"skb_set_tunnel_key", "4.3"},
+  {"perf_event_read", "4.3"},
+  {"redirect", "4.4"},
+  {"get_route_realm", "4.4"},
+  {"perf_event_output", "4.4"},
+  {"skb_load_bytes", "4.5"},
+  {"get_stackid", "4.6"},
+  {"csum_diff", "4.6"},
+  {"skb_get_tunnel_opt", "4.6"},
+  {"skb_set_tunnel_opt", "4.6"},
+  {"skb_change_proto", "4.8"},
+  {"skb_change_type", "4.8"},
+  {"skb_under_cgroup", "4.8"},
+  {"get_hash_recalc", "4.8"},
+  {"get_current_task", "4.8"},
+  {"probe_write_user", "4.8"},
+  {"current_task_under_cgroup", "4.9"},
+  {"skb_change_tail", "4.9"},
+  {"skb_pull_data", "4.9"},
+  {"csum_update", "4.9"},
+  {"set_hash_invalid", "4.9"},
+  {"get_numa_node_id", "4.10"},
+  {"skb_change_head", "4.10"},
+  {"xdp_adjust_head", "4.10"},
+  {"probe_read_str", "4.11"},
+  {"get_socket_cookie", "4.12"},
+  {"get_socket_uid", "4.12"},
+  {"set_hash", "4.13"},
+  {"setsockopt", "4.13"},
+  {"skb_adjust_room", "4.13"},
+  {"redirect_map", "4.14"},
+  {"sk_redirect_map", "4.14"},
+  {"sock_map_update", "4.14"},
+  {"xdp_adjust_meta", "4.15"},
+  {"perf_event_read_value", "4.15"},
+  {"perf_prog_read_value", "4.15"},
+  {"getsockopt", "4.15"},
+  {"override_return", "4.16"},
+  {"sock_ops_cb_flags_set", "4.16"},
+  {"msg_redirect_map", "4.17"},
+  {"msg_apply_bytes", "4.17"},
+  {"msg_cork_bytes", "4.17"},
+  {"msg_pull_data", "4.17"},
+  {"bind", "4.17"},
+  {"xdp_adjust_tail", "4.18"},
+  {"skb_get_xfrm_state", "4.18"},
+  {"get_stack", "4.18"},
+  {"skb_load_bytes_relative", "4.18"},
+  {"fib_lookup", "4.18"},
+  {"sock_hash_update", "4.18"},
+  {"msg_redirect_hash", "4.18"},
+  {"sk_redirect_hash", "4.18"},
+  {"lwt_push_encap", "4.18"},
+  {"lwt_seg6_store_bytes", "4.18"},
+  {"lwt_seg6_adjust_srh", "4.18"},
+  {"lwt_seg6_action", "4.18"},
+  {"rc_repeat", "4.18"},
+  {"rc_keydown", "4.18"},
+  {"skb_cgroup_id", "4.18"},
+  {"get_current_cgroup_id", "4.18"},
+  {"get_local_storage", "4.19"},
+  {"sk_select_reuseport", "4.19"},
+  {"skb_ancestor_cgroup_id", "4.19"},
+  {"sk_lookup_tcp", "4.20"},
+  {"sk_lookup_udp", "4.20"},
+  {"sk_release", "4.20"},
+  {"map_push_elem", "4.20"},
+  {"map_pop_elem", "4.20"},
+  {"map_peak_elem", "4.20"},
+  {"msg_push_data", "4.20"},
+  {"msg_pop_data", "4.21"},
+  {"rc_pointer_rel", "4.21"},
+};
 
 static uint64_t ptr_to_u64(void *ptr)
 {
@@ -237,6 +343,30 @@ static void bpf_print_hints(int ret, char *log)
       "bpf_probe_read is automatic by the bcc rewriter, other times "
       "you'll need to be explicit.\n\n");
   }
+
+  // referencing global/static variables or read only data
+  if (strstr(log, "unknown opcode") != NULL) {
+    fprintf(stderr, "HINT: The 'unknown opcode' can happen if you reference "
+      "a global or static variable, or data in read-only section. For example,"
+      " 'char *p = \"hello\"' will result in p referencing a read-only section,"
+      " and 'char p[] = \"hello\"' will have \"hello\" stored on the stack.\n\n");
+  }
+
+  // helper function not found in kernel
+  char *helper_str = strstr(log, "invalid func ");
+  if (helper_str != NULL) {
+    helper_str += strlen("invalid func ");
+    char *str = strchr(helper_str, '#');
+    if (str != NULL) {
+      helper_str = str + 1;
+    }
+    int helper_id = atoi(helper_str);
+    if (helper_id && helper_id < sizeof(helpers) / sizeof(struct bpf_helper)) {
+      struct bpf_helper helper = helpers[helper_id - 1];
+      fprintf(stderr, "HINT: bpf_%s missing (added in Linux %s).\n\n",
+              helper.name, helper.required_version);
+    }
+  }
 }
 #define ROUND_UP(x, n) (((x) + (n) - 1u) & ~((n) - 1u))
 
@@ -319,6 +449,8 @@ int bpf_prog_compute_tag(const struct bpf_insn *insns, int prog_len,
     return -1;
   }
   *ptag = __builtin_bswap64(u.tag);
+  close(shafd2);
+  close(shafd);
   return 0;
 }
 
@@ -358,7 +490,7 @@ int bpf_prog_load(enum bpf_prog_type prog_type, const char *name,
   union bpf_attr attr;
   char *tmp_log_buf = NULL;
   unsigned tmp_log_buf_size = 0;
-  int ret = 0;
+  int ret = 0, name_offset = 0;
 
   memset(&attr, 0, sizeof(attr));
 
@@ -371,8 +503,8 @@ int bpf_prog_load(enum bpf_prog_type prog_type, const char *name,
   if (attr.insn_cnt > BPF_MAXINSNS) {
     errno = EINVAL;
     fprintf(stderr,
-            "bpf: %s. Program too large (%u insns), at most %d insns\n\n",
-            strerror(errno), attr.insn_cnt, BPF_MAXINSNS);
+            "bpf: %s. Program %s too large (%u insns), at most %d insns\n\n",
+            strerror(errno), name, attr.insn_cnt, BPF_MAXINSNS);
     return -1;
   }
 
@@ -399,7 +531,16 @@ int bpf_prog_load(enum bpf_prog_type prog_type, const char *name,
     }
   }
 
-  memcpy(attr.prog_name, name, min(name_len, BPF_OBJ_NAME_LEN - 1));
+  if (name_len) {
+    if (strncmp(name, "kprobe__", 8) == 0)
+      name_offset = 8;
+    else if (strncmp(name, "tracepoint__", 12) == 0)
+      name_offset = 12;
+    else if (strncmp(name, "raw_tracepoint__", 16) == 0)
+      name_offset = 16;
+    memcpy(attr.prog_name, name + name_offset,
+           min(name_len - name_offset, BPF_OBJ_NAME_LEN - 1));
+  }
 
   ret = syscall(__NR_bpf, BPF_PROG_LOAD, &attr, sizeof(attr));
   // BPF object name is not supported on older Kernels.
@@ -474,10 +615,12 @@ int bpf_prog_load(enum bpf_prog_type prog_type, const char *name,
     }
   }
 
-  // If log_level is not 0, either speficied by user or set due to error,
-  // print the log message.
+  // Check if we should print the log message if log_level is not 0,
+  // either specified by user or set due to error.
   if (attr.log_level > 0) {
-    if (log_buf)
+    // Don't print if user enabled logging and provided log buffer,
+    // but there is no error.
+    if (log_buf && ret < 0)
       bpf_print_hints(ret, log_buf);
     else if (tmp_log_buf)
       bpf_print_hints(ret, tmp_log_buf);
@@ -526,48 +669,166 @@ int bpf_attach_socket(int sock, int prog) {
   return setsockopt(sock, SOL_SOCKET, SO_ATTACH_BPF, &prog, sizeof(prog));
 }
 
-static int bpf_attach_tracing_event(int progfd, const char *event_path,
-    struct perf_reader *reader, int pid, int cpu, int group_fd) {
-  int efd, pfd;
-  ssize_t bytes;
-  char buf[256];
+#define PMU_TYPE_FILE "/sys/bus/event_source/devices/%s/type"
+static int bpf_find_probe_type(const char *event_type)
+{
+  int fd;
+  int ret;
+  char buf[PATH_MAX];
+
+  ret = snprintf(buf, sizeof(buf), PMU_TYPE_FILE, event_type);
+  if (ret < 0 || ret >= sizeof(buf))
+    return -1;
+
+  fd = open(buf, O_RDONLY);
+  if (fd < 0)
+    return -1;
+  ret = read(fd, buf, sizeof(buf));
+  close(fd);
+  if (ret < 0 || ret >= sizeof(buf))
+    return -1;
+  errno = 0;
+  ret = (int)strtol(buf, NULL, 10);
+  return errno ? -1 : ret;
+}
+
+#define PMU_RETPROBE_FILE "/sys/bus/event_source/devices/%s/format/retprobe"
+static int bpf_get_retprobe_bit(const char *event_type)
+{
+  int fd;
+  int ret;
+  char buf[PATH_MAX];
+
+  ret = snprintf(buf, sizeof(buf), PMU_RETPROBE_FILE, event_type);
+  if (ret < 0 || ret >= sizeof(buf))
+    return -1;
+
+  fd = open(buf, O_RDONLY);
+  if (fd < 0)
+    return -1;
+  ret = read(fd, buf, sizeof(buf));
+  close(fd);
+  if (ret < 0 || ret >= sizeof(buf))
+    return -1;
+  if (strncmp(buf, "config:", strlen("config:")))
+    return -1;
+  errno = 0;
+  ret = (int)strtol(buf + strlen("config:"), NULL, 10);
+  return errno ? -1 : ret;
+}
+
+/*
+ * new kernel API allows creating [k,u]probe with perf_event_open, which
+ * makes it easier to clean up the [k,u]probe. This function tries to
+ * create pfd with the new API.
+ */
+static int bpf_try_perf_event_open_with_probe(const char *name, uint64_t offs,
+             int pid, char *event_type, int is_return)
+{
   struct perf_event_attr attr = {};
+  int type = bpf_find_probe_type(event_type);
+  int is_return_bit = bpf_get_retprobe_bit(event_type);
+  int cpu = 0;
 
-  snprintf(buf, sizeof(buf), "%s/id", event_path);
-  efd = open(buf, O_RDONLY, 0);
-  if (efd < 0) {
-    fprintf(stderr, "open(%s): %s\n", buf, strerror(errno));
+  if (type < 0 || is_return_bit < 0)
     return -1;
-  }
-
-  bytes = read(efd, buf, sizeof(buf));
-  if (bytes <= 0 || bytes >= sizeof(buf)) {
-    fprintf(stderr, "read(%s): %s\n", buf, strerror(errno));
-    close(efd);
-    return -1;
-  }
-  close(efd);
-  buf[bytes] = '\0';
-  attr.config = strtol(buf, NULL, 0);
-  attr.type = PERF_TYPE_TRACEPOINT;
-  attr.sample_type = PERF_SAMPLE_RAW | PERF_SAMPLE_CALLCHAIN;
   attr.sample_period = 1;
   attr.wakeup_events = 1;
-  pfd = syscall(__NR_perf_event_open, &attr, pid, cpu, group_fd, PERF_FLAG_FD_CLOEXEC);
-  if (pfd < 0) {
-    fprintf(stderr, "perf_event_open(%s/id): %s\n", event_path, strerror(errno));
-    return -1;
+  if (is_return)
+    attr.config |= 1 << is_return_bit;
+
+  /*
+   * struct perf_event_attr in latest perf_event.h has the following
+   * extension to config1 and config2. To keep bcc compatibe with
+   * older perf_event.h, we use config1 and config2 here instead of
+   * kprobe_func, uprobe_path, kprobe_addr, and probe_offset.
+   *
+   * union {
+   *  __u64 bp_addr;
+   *  __u64 kprobe_func;
+   *  __u64 uprobe_path;
+   *  __u64 config1;
+   * };
+   * union {
+   *   __u64 bp_len;
+   *   __u64 kprobe_addr;
+   *   __u64 probe_offset;
+   *   __u64 config2;
+   * };
+   */
+  attr.config2 = offs;  /* config2 here is kprobe_addr or probe_offset */
+  attr.size = sizeof(attr);
+  attr.type = type;
+  /* config1 here is kprobe_func or  uprobe_path */
+  attr.config1 = ptr_to_u64((void *)name);
+  // PID filter is only possible for uprobe events.
+  if (pid < 0)
+    pid = -1;
+  // perf_event_open API doesn't allow both pid and cpu to be -1.
+  // So only set it to -1 when PID is not -1.
+  // Tracing events do not do CPU filtering in any cases.
+  if (pid != -1)
+    cpu = -1;
+  return syscall(__NR_perf_event_open, &attr, pid, cpu, -1 /* group_fd */,
+                 PERF_FLAG_FD_CLOEXEC);
+}
+
+// When a valid Perf Event FD provided through pfd, it will be used to enable
+// and attach BPF program to the event, and event_path will be ignored.
+// Otherwise, event_path is expected to contain the path to the event in debugfs
+// and it will be used to open the Perf Event FD.
+// In either case, if the attach partially failed (such as issue with the
+// ioctl operations), the **caller** need to clean up the Perf Event FD, either
+// provided by the caller or opened here.
+static int bpf_attach_tracing_event(int progfd, const char *event_path, int pid,
+                                    int *pfd)
+{
+  int efd, cpu = 0;
+  ssize_t bytes;
+  char buf[PATH_MAX];
+  struct perf_event_attr attr = {};
+  // Caller did not provided a valid Perf Event FD. Create one with the debugfs
+  // event path provided.
+  if (*pfd < 0) {
+    snprintf(buf, sizeof(buf), "%s/id", event_path);
+    efd = open(buf, O_RDONLY, 0);
+    if (efd < 0) {
+      fprintf(stderr, "open(%s): %s\n", buf, strerror(errno));
+      return -1;
+    }
+
+    bytes = read(efd, buf, sizeof(buf));
+    if (bytes <= 0 || bytes >= sizeof(buf)) {
+      fprintf(stderr, "read(%s): %s\n", buf, strerror(errno));
+      close(efd);
+      return -1;
+    }
+    close(efd);
+    buf[bytes] = '\0';
+    attr.config = strtol(buf, NULL, 0);
+    attr.type = PERF_TYPE_TRACEPOINT;
+    attr.sample_period = 1;
+    attr.wakeup_events = 1;
+    // PID filter is only possible for uprobe events.
+    if (pid < 0)
+      pid = -1;
+    // perf_event_open API doesn't allow both pid and cpu to be -1.
+    // So only set it to -1 when PID is not -1.
+    // Tracing events do not do CPU filtering in any cases.
+    if (pid != -1)
+      cpu = -1;
+    *pfd = syscall(__NR_perf_event_open, &attr, pid, cpu, -1 /* group_fd */, PERF_FLAG_FD_CLOEXEC);
+    if (*pfd < 0) {
+      fprintf(stderr, "perf_event_open(%s/id): %s\n", event_path, strerror(errno));
+      return -1;
+    }
   }
-  perf_reader_set_fd(reader, pfd);
 
-  if (perf_reader_mmap(reader, attr.type, attr.sample_type) < 0)
-    return -1;
-
-  if (ioctl(pfd, PERF_EVENT_IOC_SET_BPF, progfd) < 0) {
+  if (ioctl(*pfd, PERF_EVENT_IOC_SET_BPF, progfd) < 0) {
     perror("ioctl(PERF_EVENT_IOC_SET_BPF)");
     return -1;
   }
-  if (ioctl(pfd, PERF_EVENT_IOC_ENABLE, 0) < 0) {
+  if (ioctl(*pfd, PERF_EVENT_IOC_ENABLE, 0) < 0) {
     perror("ioctl(PERF_EVENT_IOC_ENABLE)");
     return -1;
   }
@@ -575,49 +836,58 @@ static int bpf_attach_tracing_event(int progfd, const char *event_path,
   return 0;
 }
 
-void * bpf_attach_kprobe(int progfd, enum bpf_probe_attach_type attach_type, const char *ev_name,
-                        const char *fn_name,
-                        pid_t pid, int cpu, int group_fd,
-                        perf_reader_cb cb, void *cb_cookie)
+int bpf_attach_kprobe(int progfd, enum bpf_probe_attach_type attach_type,
+                      const char *ev_name, const char *fn_name, uint64_t fn_offset)
 {
-  int kfd;
+  int kfd, pfd = -1;
   char buf[256];
   char event_alias[128];
-  struct perf_reader *reader = NULL;
   static char *event_type = "kprobe";
 
-  reader = perf_reader_new(cb, NULL, NULL, cb_cookie, probe_perf_reader_page_cnt);
-  if (!reader)
-    goto error;
+  // Try create the kprobe Perf Event with perf_event_open API.
+  pfd = bpf_try_perf_event_open_with_probe(fn_name, fn_offset, -1, event_type,
+                                           attach_type != BPF_PROBE_ENTRY);
+  // If failed, most likely Kernel doesn't support the new perf_event_open API
+  // yet. Try create the event using debugfs.
+  if (pfd < 0) {
+    snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/%s_events", event_type);
+    kfd = open(buf, O_WRONLY | O_APPEND, 0);
+    if (kfd < 0) {
+      fprintf(stderr, "open(%s): %s\n", buf, strerror(errno));
+      goto error;
+    }
 
-  snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/%s_events", event_type);
-  kfd = open(buf, O_WRONLY | O_APPEND, 0);
-  if (kfd < 0) {
-    fprintf(stderr, "open(%s): %s\n", buf, strerror(errno));
-    goto error;
-  }
+    snprintf(event_alias, sizeof(event_alias), "%s_bcc_%d", ev_name, getpid());
 
-  snprintf(event_alias, sizeof(event_alias), "%s_bcc_%d", ev_name, getpid());
-  snprintf(buf, sizeof(buf), "%c:%ss/%s %s", attach_type==BPF_PROBE_ENTRY ? 'p' : 'r',
-			event_type, event_alias, fn_name);
-  if (write(kfd, buf, strlen(buf)) < 0) {
-    if (errno == EINVAL)
-      fprintf(stderr, "check dmesg output for possible cause\n");
+    if (fn_offset > 0 && attach_type == BPF_PROBE_ENTRY)
+      snprintf(buf, sizeof(buf), "p:%ss/%s %s+%"PRIu64,
+               event_type, event_alias, fn_name, fn_offset);
+    else
+      snprintf(buf, sizeof(buf), "%c:%ss/%s %s",
+               attach_type == BPF_PROBE_ENTRY ? 'p' : 'r',
+               event_type, event_alias, fn_name);
+
+    if (write(kfd, buf, strlen(buf)) < 0) {
+      if (errno == ENOENT)
+         fprintf(stderr, "cannot attach kprobe, probe entry may not exist\n");
+      else
+         fprintf(stderr, "cannot attach kprobe, %s\n", strerror(errno));
+      close(kfd);
+      goto error;
+    }
     close(kfd);
-    goto error;
+    snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/events/%ss/%s", event_type, event_alias);
   }
-  close(kfd);
-
-  snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/events/%ss/%s", event_type, event_alias);
-  if (bpf_attach_tracing_event(progfd, buf, reader, pid, cpu, group_fd) < 0)
-    goto error;
-
-  return reader;
+  // If perf_event_open succeeded, bpf_attach_tracing_event will use the created
+  // Perf Event FD directly and buf would be empty and unused.
+  // Otherwise it will read the event ID from the path in buf, create the
+  // Perf Event event using that ID, and updated value of pfd.
+  if (bpf_attach_tracing_event(progfd, buf, -1 /* PID */, &pfd) == 0)
+    return pfd;
 
 error:
-  perf_reader_free(reader);
-  return NULL;
-
+  bpf_close_perf_event_fd(pfd);
+  return -1;
 }
 
 static int enter_mount_ns(int pid) {
@@ -679,70 +949,110 @@ static void exit_mount_ns(int fd) {
 
   if (setns(fd, CLONE_NEWNS))
     perror("setns");
+  close(fd);
 }
 
-void * bpf_attach_uprobe(int progfd, enum bpf_probe_attach_type attach_type, const char *ev_name,
-                        const char *binary_path, uint64_t offset,
-                        pid_t pid, int cpu, int group_fd,
-                        perf_reader_cb cb, void *cb_cookie)
+int bpf_attach_uprobe(int progfd, enum bpf_probe_attach_type attach_type,
+                      const char *ev_name, const char *binary_path,
+                      uint64_t offset, pid_t pid)
 {
   char buf[PATH_MAX];
   char event_alias[PATH_MAX];
-  struct perf_reader *reader = NULL;
   static char *event_type = "uprobe";
-  int res, kfd = -1, ns_fd = -1;
+  int res, kfd = -1, pfd = -1, ns_fd = -1;
+  // Try create the uprobe Perf Event with perf_event_open API.
+  pfd = bpf_try_perf_event_open_with_probe(binary_path, offset, pid, event_type,
+                                           attach_type != BPF_PROBE_ENTRY);
+  // If failed, most likely Kernel doesn't support the new perf_event_open API
+  // yet. Try create the event using debugfs.
+  if (pfd < 0) {
+    snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/%s_events", event_type);
+    kfd = open(buf, O_WRONLY | O_APPEND, 0);
+    if (kfd < 0) {
+      fprintf(stderr, "open(%s): %s\n", buf, strerror(errno));
+      goto error;
+    }
 
-  reader = perf_reader_new(cb, NULL, NULL, cb_cookie, probe_perf_reader_page_cnt);
-  if (!reader)
-    goto error;
+    res = snprintf(event_alias, sizeof(event_alias), "%s_bcc_%d", ev_name, getpid());
+    if (res < 0 || res >= sizeof(event_alias)) {
+      fprintf(stderr, "Event name (%s) is too long for buffer\n", ev_name);
+      goto error;
+    }
+    res = snprintf(buf, sizeof(buf), "%c:%ss/%s %s:0x%lx", attach_type==BPF_PROBE_ENTRY ? 'p' : 'r',
+                   event_type, event_alias, binary_path, offset);
+    if (res < 0 || res >= sizeof(buf)) {
+      fprintf(stderr, "Event alias (%s) too long for buffer\n", event_alias);
+      goto error;
+    }
 
-  snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/%s_events", event_type);
-  kfd = open(buf, O_WRONLY | O_APPEND, 0);
-  if (kfd < 0) {
-    fprintf(stderr, "open(%s): %s\n", buf, strerror(errno));
-    goto error;
+    ns_fd = enter_mount_ns(pid);
+    if (write(kfd, buf, strlen(buf)) < 0) {
+      if (errno == EINVAL)
+        fprintf(stderr, "check dmesg output for possible cause\n");
+      goto error;
+    }
+    close(kfd);
+    kfd = -1;
+    exit_mount_ns(ns_fd);
+    ns_fd = -1;
+
+    snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/events/%ss/%s", event_type, event_alias);
   }
-
-  res = snprintf(event_alias, sizeof(event_alias), "%s_bcc_%d", ev_name, getpid());
-  if (res < 0 || res >= sizeof(event_alias)) {
-    fprintf(stderr, "Event name (%s) is too long for buffer\n", ev_name);
-    goto error;
-  }
-  res = snprintf(buf, sizeof(buf), "%c:%ss/%s %s:0x%lx", attach_type==BPF_PROBE_ENTRY ? 'p' : 'r',
-			event_type, event_alias, binary_path, offset);
-  if (res < 0 || res >= sizeof(buf)) {
-    fprintf(stderr, "Event alias (%s) too long for buffer\n", event_alias);
-    goto error;
-  }
-
-  ns_fd = enter_mount_ns(pid);
-  if (write(kfd, buf, strlen(buf)) < 0) {
-    if (errno == EINVAL)
-      fprintf(stderr, "check dmesg output for possible cause\n");
-    goto error;
-  }
-  close(kfd);
-  exit_mount_ns(ns_fd);
-  ns_fd = -1;
-
-  snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/events/%ss/%s", event_type, event_alias);
-  if (bpf_attach_tracing_event(progfd, buf, reader, pid, cpu, group_fd) < 0)
-    goto error;
-
-  return reader;
+  // If perf_event_open succeeded, bpf_attach_tracing_event will use the created
+  // Perf Event FD directly and buf would be empty and unused.
+  // Otherwise it will read the event ID from the path in buf, create the
+  // Perf Event event using that ID, and updated value of pfd.
+  if (bpf_attach_tracing_event(progfd, buf, pid, &pfd) == 0)
+    return pfd;
 
 error:
   if (kfd >= 0)
     close(kfd);
   exit_mount_ns(ns_fd);
-  perf_reader_free(reader);
-  return NULL;
+  bpf_close_perf_event_fd(pfd);
+  return -1;
 }
 
 static int bpf_detach_probe(const char *ev_name, const char *event_type)
 {
-  int kfd, res;
+  int kfd = -1, res;
   char buf[PATH_MAX];
+  int found_event = 0;
+  size_t bufsize = 0;
+  char *cptr = NULL;
+  FILE *fp;
+
+  /*
+   * For [k,u]probe created with perf_event_open (on newer kernel), it is
+   * not necessary to clean it up in [k,u]probe_events. We first look up
+   * the %s_bcc_%d line in [k,u]probe_events. If the event is not found,
+   * it is safe to skip the cleaning up process (write -:... to the file).
+   */
+  snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/%s_events", event_type);
+  fp = fopen(buf, "r");
+  if (!fp) {
+    fprintf(stderr, "open(%s): %s\n", buf, strerror(errno));
+    goto error;
+  }
+
+  res = snprintf(buf, sizeof(buf), "%ss/%s_bcc_%d", event_type, ev_name, getpid());
+  if (res < 0 || res >= sizeof(buf)) {
+    fprintf(stderr, "snprintf(%s): %d\n", ev_name, res);
+    goto error;
+  }
+
+  while (getline(&cptr, &bufsize, fp) != -1)
+    if (strstr(cptr, buf) != NULL) {
+      found_event = 1;
+      break;
+    }
+  free(cptr);
+  fclose(fp);
+  fp = NULL;
+
+  if (!found_event)
+    return 0;
+
   snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/%s_events", event_type);
   kfd = open(buf, O_WRONLY | O_APPEND, 0);
   if (kfd < 0) {
@@ -766,6 +1076,8 @@ static int bpf_detach_probe(const char *ev_name, const char *event_type)
 error:
   if (kfd >= 0)
     close(kfd);
+  if (fp)
+    fclose(fp);
   return -1;
 }
 
@@ -780,26 +1092,19 @@ int bpf_detach_uprobe(const char *ev_name)
 }
 
 
-void * bpf_attach_tracepoint(int progfd, const char *tp_category,
-                             const char *tp_name, int pid, int cpu,
-                             int group_fd, perf_reader_cb cb, void *cb_cookie) {
+int bpf_attach_tracepoint(int progfd, const char *tp_category,
+                          const char *tp_name)
+{
   char buf[256];
-  struct perf_reader *reader = NULL;
-
-  reader = perf_reader_new(cb, NULL, NULL, cb_cookie, probe_perf_reader_page_cnt);
-  if (!reader)
-    goto error;
+  int pfd = -1;
 
   snprintf(buf, sizeof(buf), "/sys/kernel/debug/tracing/events/%s/%s",
            tp_category, tp_name);
-  if (bpf_attach_tracing_event(progfd, buf, reader, pid, cpu, group_fd) < 0)
-    goto error;
+  if (bpf_attach_tracing_event(progfd, buf, -1 /* PID */, &pfd) == 0)
+    return pfd;
 
-  return reader;
-
-error:
-  perf_reader_free(reader);
-  return NULL;
+  bpf_close_perf_event_fd(pfd);
+  return -1;
 }
 
 int bpf_detach_tracepoint(const char *tp_category, const char *tp_name) {
@@ -808,6 +1113,21 @@ int bpf_detach_tracepoint(const char *tp_category, const char *tp_name) {
   return 0;
 }
 
+int bpf_attach_raw_tracepoint(int progfd, char *tp_name)
+{
+  union bpf_attr attr;
+  int ret;
+
+  bzero(&attr, sizeof(attr));
+  attr.raw_tracepoint.name = ptr_to_u64(tp_name);
+  attr.raw_tracepoint.prog_fd = progfd;
+
+  ret = syscall(__NR_bpf, BPF_RAW_TRACEPOINT_OPEN, &attr, sizeof(attr));
+  if (ret < 0)
+    fprintf(stderr, "bpf_attach_raw_tracepoint (%s): %s\n", tp_name, strerror(errno));
+  return ret;
+}
+
 void * bpf_open_perf_buffer(perf_reader_raw_cb raw_cb,
                             perf_reader_lost_cb lost_cb, void *cb_cookie,
                             int pid, int cpu, int page_cnt) {
@@ -815,7 +1135,7 @@ void * bpf_open_perf_buffer(perf_reader_raw_cb raw_cb,
   struct perf_event_attr attr = {};
   struct perf_reader *reader = NULL;
 
-  reader = perf_reader_new(NULL, raw_cb, lost_cb, cb_cookie, page_cnt);
+  reader = perf_reader_new(raw_cb, lost_cb, cb_cookie, page_cnt);
   if (!reader)
     goto error;
 
@@ -832,7 +1152,7 @@ void * bpf_open_perf_buffer(perf_reader_raw_cb raw_cb,
   }
   perf_reader_set_fd(reader, pfd);
 
-  if (perf_reader_mmap(reader, attr.type, attr.sample_type) < 0)
+  if (perf_reader_mmap(reader) < 0)
     goto error;
 
   if (ioctl(pfd, PERF_EVENT_IOC_ENABLE, 0) < 0) {
@@ -1036,33 +1356,10 @@ cleanup:
     return ret;
 }
 
-int bpf_attach_perf_event(int progfd, uint32_t ev_type, uint32_t ev_config,
-                          uint64_t sample_period, uint64_t sample_freq,
-                          pid_t pid, int cpu, int group_fd) {
-  if (invalid_perf_config(ev_type, ev_config)) {
-    return -1;
-  }
-  if (!((sample_period > 0) ^ (sample_freq > 0))) {
-    fprintf(
-      stderr, "Exactly one of sample_period / sample_freq should be set\n"
-    );
-    return -1;
-  }
-
-  struct perf_event_attr attr = {};
-  attr.type = ev_type;
-  attr.config = ev_config;
-  attr.inherit = 1;
-  if (sample_freq > 0) {
-    attr.freq = 1;
-    attr.sample_freq = sample_freq;
-  } else {
-    attr.sample_period = sample_period;
-  }
-
-  int fd = syscall(
-    __NR_perf_event_open, &attr, pid, cpu, group_fd, PERF_FLAG_FD_CLOEXEC
-  );
+int bpf_attach_perf_event_raw(int progfd, void *perf_event_attr, pid_t pid,
+                              int cpu, int group_fd, unsigned long extra_flags) {
+  int fd = syscall(__NR_perf_event_open, perf_event_attr, pid, cpu, group_fd,
+                   PERF_FLAG_FD_CLOEXEC | extra_flags);
   if (fd < 0) {
     perror("perf_event_open failed");
     return -1;
@@ -1081,6 +1378,34 @@ int bpf_attach_perf_event(int progfd, uint32_t ev_type, uint32_t ev_config,
   return fd;
 }
 
+int bpf_attach_perf_event(int progfd, uint32_t ev_type, uint32_t ev_config,
+                          uint64_t sample_period, uint64_t sample_freq,
+                          pid_t pid, int cpu, int group_fd) {
+  if (invalid_perf_config(ev_type, ev_config)) {
+    return -1;
+  }
+  if (!((sample_period > 0) ^ (sample_freq > 0))) {
+    fprintf(
+      stderr, "Exactly one of sample_period / sample_freq should be set\n"
+    );
+    return -1;
+  }
+
+  struct perf_event_attr attr = {};
+  attr.type = ev_type;
+  attr.config = ev_config;
+  if (pid > 0)
+    attr.inherit = 1;
+  if (sample_freq > 0) {
+    attr.freq = 1;
+    attr.sample_freq = sample_freq;
+  } else {
+    attr.sample_period = sample_period;
+  }
+
+  return bpf_attach_perf_event_raw(progfd, &attr, pid, cpu, group_fd, 0);
+}
+
 int bpf_close_perf_event_fd(int fd) {
   int res, error = 0;
   if (fd >= 0) {
