@@ -36,6 +36,13 @@ BPF_MAP_TYPE_CGROUP_ARRAY = 8
 BPF_MAP_TYPE_LRU_HASH = 9
 BPF_MAP_TYPE_LRU_PERCPU_HASH = 10
 BPF_MAP_TYPE_LPM_TRIE = 11
+BPF_MAP_TYPE_ARRAY_OF_MAPS = 12
+BPF_MAP_TYPE_HASH_OF_MAPS = 13
+BPF_MAP_TYPE_DEVMAP = 14
+BPF_MAP_TYPE_SOCKMAP = 15
+BPF_MAP_TYPE_CPUMAP = 16
+BPF_MAP_TYPE_XSKMAP = 17
+BPF_MAP_TYPE_SOCKHASH = 18
 
 stars_max = 40
 log2_index_max = 65
@@ -142,6 +149,12 @@ def Table(bpf, map_id, map_fd, keytype, leaftype, **kwargs):
         t = LruHash(bpf, map_id, map_fd, keytype, leaftype)
     elif ttype == BPF_MAP_TYPE_LRU_PERCPU_HASH:
         t = LruPerCpuHash(bpf, map_id, map_fd, keytype, leaftype)
+    elif ttype == BPF_MAP_TYPE_CGROUP_ARRAY:
+        t = CgroupArray(bpf, map_id, map_fd, keytype, leaftype)
+    elif ttype == BPF_MAP_TYPE_DEVMAP:
+        t = DevMap(bpf, map_id, map_fd, keytype, leaftype)
+    elif ttype == BPF_MAP_TYPE_CPUMAP:
+        t = CpuMap(bpf, map_id, map_fd, keytype, leaftype)
     if t == None:
         raise Exception("Unknown table type %d" % ttype)
     return t
@@ -199,12 +212,16 @@ class TableBase(MutableMapping):
         return leaf
 
     def __setitem__(self, key, leaf):
-        res = lib.bpf_update_elem(self.map_fd, ct.byref(key), ct.byref(leaf),
-                                  0)
+        res = lib.bpf_update_elem(self.map_fd, ct.byref(key), ct.byref(leaf), 0)
         if res < 0:
             errstr = os.strerror(ct.get_errno())
             raise Exception("Could not update table: %s" % errstr)
 
+    def __delitem__(self, key):
+        res = lib.bpf_delete_elem(self.map_fd, ct.byref(key))
+        if res < 0:
+            raise KeyError
+
     # override the MutableMapping's implementation of these since they
     # don't handle KeyError nicely
     def itervalues(self):
@@ -275,9 +292,11 @@ class TableBase(MutableMapping):
         return next_key
 
     def print_log2_hist(self, val_type="value", section_header="Bucket ptr",
-            section_print_fn=None, bucket_fn=None, strip_leading_zero=None):
+            section_print_fn=None, bucket_fn=None, strip_leading_zero=None,
+            bucket_sort_fn=None):
         """print_log2_hist(val_type="value", section_header="Bucket ptr",
-                           section_print_fn=None, bucket_fn=None)
+                           section_print_fn=None, bucket_fn=None,
+                           strip_leading_zero=None, bucket_sort_fn=None):
 
         Prints a table as a log2 histogram. The table must be stored as
         log2. The val_type argument is optional, and is a column header.
@@ -287,9 +306,12 @@ class TableBase(MutableMapping):
         to format into a string as it sees fit. If bucket_fn is not None,
         it will be used to produce a bucket value for the histogram keys.
         If the value of strip_leading_zero is not False, prints a histogram
-        that is omitted leading zeros from the beginning. The maximum index
-        allowed is log2_index_max (65), which will accommodate any 64-bit
-        integer in the histogram.
+        that is omitted leading zeros from the beginning.
+        If bucket_sort_fn is not None, it will be used to sort the buckets
+        before iterating them, and it is useful when there are multiple fields
+        in the secondary key.
+        The maximum index allowed is log2_index_max (65), which will
+        accommodate any 64-bit integer in the histogram.
         """
         if isinstance(self.Key(), ct.Structure):
             tmp = {}
@@ -302,7 +324,13 @@ class TableBase(MutableMapping):
                 vals = tmp[bucket] = tmp.get(bucket, [0] * log2_index_max)
                 slot = getattr(k, f2)
                 vals[slot] = v.value
-            for bucket, vals in tmp.items():
+
+            buckets = list(tmp.keys())
+            if bucket_sort_fn:
+                buckets = bucket_sort_fn(buckets)
+
+            for bucket in buckets:
+                vals = tmp[bucket]
                 if section_print_fn:
                     print("\n%s = %s" % (section_header,
                         section_print_fn(bucket)))
@@ -316,9 +344,10 @@ class TableBase(MutableMapping):
             _print_log2_hist(vals, val_type, strip_leading_zero)
 
     def print_linear_hist(self, val_type="value", section_header="Bucket ptr",
-            section_print_fn=None, bucket_fn=None):
+            section_print_fn=None, bucket_fn=None, bucket_sort_fn=None):
         """print_linear_hist(val_type="value", section_header="Bucket ptr",
-                           section_print_fn=None, bucket_fn=None)
+                           section_print_fn=None, bucket_fn=None,
+                           bucket_sort_fn=None)
 
         Prints a table as a linear histogram. This is intended to span integer
         ranges, eg, from 0 to 100. The val_type argument is optional, and is a
@@ -327,6 +356,9 @@ class TableBase(MutableMapping):
         each.  If section_print_fn is not None, it will be passed the bucket
         value to format into a string as it sees fit. If bucket_fn is not None,
         it will be used to produce a bucket value for the histogram keys.
+        If bucket_sort_fn is not None, it will be used to sort the buckets
+        before iterating them, and it is useful when there are multiple fields
+        in the secondary key.
         The maximum index allowed is linear_index_max (1025), which is hoped
         to be sufficient for integer ranges spanned.
         """
@@ -341,7 +373,13 @@ class TableBase(MutableMapping):
                 vals = tmp[bucket] = tmp.get(bucket, [0] * linear_index_max)
                 slot = getattr(k, f2)
                 vals[slot] = v.value
-            for bucket, vals in tmp.items():
+
+            buckets = tmp.keys()
+            if bucket_sort_fn:
+                buckets = bucket_sort_fn(buckets)
+
+            for bucket in buckets:
+                vals = tmp[bucket]
                 if section_print_fn:
                     print("\n%s = %s" % (section_header,
                         section_print_fn(bucket)))
@@ -370,11 +408,6 @@ class HashTable(TableBase):
         for k in self: i += 1
         return i
 
-    def __delitem__(self, key):
-        res = lib.bpf_delete_elem(self.map_fd, ct.byref(key))
-        if res < 0:
-            raise KeyError
-
 class LruHash(HashTable):
     def __init__(self, *args, **kwargs):
         super(LruHash, self).__init__(*args, **kwargs)
@@ -409,11 +442,12 @@ class ArrayBase(TableBase):
 
     def __delitem__(self, key):
         key = self._normalize_key(key)
-        # Deleting from array type maps does not have an effect, so
-        # zero out the entry instead.
+        super(ArrayBase, self).__delitem__(key)
+
+    def clearitem(self, key):
+        key = self._normalize_key(key)
         leaf = self.Leaf()
-        res = lib.bpf_update_elem(self.map_fd, ct.byref(key), ct.byref(leaf),
-                                  0)
+        res = lib.bpf_update_elem(self.map_fd, ct.byref(key), ct.byref(leaf), 0)
         if res < 0:
             raise Exception("Could not clear item")
 
@@ -440,6 +474,9 @@ class Array(ArrayBase):
     def __init__(self, *args, **kwargs):
         super(Array, self).__init__(*args, **kwargs)
 
+    def __delitem__(self, key):
+        # Delete in Array type does not have an effect, so zero out instead
+        self.clearitem(key)
 
 class ProgArray(ArrayBase):
     def __init__(self, *args, **kwargs):
@@ -452,11 +489,39 @@ class ProgArray(ArrayBase):
             leaf = self.Leaf(leaf.fd)
         super(ProgArray, self).__setitem__(key, leaf)
 
-    def __delitem__(self, key):
-        key = self._normalize_key(key)
-        res = lib.bpf_delete_elem(self.map_fd, ct.byref(key))
-        if res < 0:
-            raise Exception("Could not delete item")
+class FileDesc:
+    def __init__(self, fd):
+        if (fd is None) or (fd < 0):
+            raise Exception("Invalid file descriptor")
+        self.fd = fd
+
+    def clean_up(self):
+        if (self.fd is not None) and (self.fd >= 0):
+            os.close(self.fd)
+            self.fd = None
+
+    def __del__(self):
+        self.clean_up()
+
+    def __enter__(self, *args, **kwargs):
+        return self
+
+    def __exit__(self, *args, **kwargs):
+        self.clean_up()
+
+class CgroupArray(ArrayBase):
+    def __init__(self, *args, **kwargs):
+        super(CgroupArray, self).__init__(*args, **kwargs)
+
+    def __setitem__(self, key, leaf):
+        if isinstance(leaf, int):
+            super(CgroupArray, self).__setitem__(key, self.Leaf(leaf))
+        elif isinstance(leaf, str):
+            # TODO: Add os.O_CLOEXEC once we move to Python version >3.3
+            with FileDesc(os.open(leaf, os.O_RDONLY)) as f:
+                super(CgroupArray, self).__setitem__(key, self.Leaf(f.fd))
+        else:
+            raise Exception("Cgroup array key must be either FD or cgroup path")
 
 class PerfEventArray(ArrayBase):
 
@@ -473,13 +538,12 @@ class PerfEventArray(ArrayBase):
         if key not in self._open_key_fds:
             return
         # Delete entry from the array
-        c_key = self._normalize_key(key)
-        lib.bpf_delete_elem(self.map_fd, ct.byref(c_key))
+        super(PerfEventArray, self).__delitem__(key)
         key_id = (id(self), key)
-        if key_id in self.bpf.open_kprobes:
+        if key_id in self.bpf.perf_buffers:
             # The key is opened for perf ring buffer
-            lib.perf_reader_free(self.bpf.open_kprobes[key_id])
-            self.bpf._del_kprobe(key_id)
+            lib.perf_reader_free(self.bpf.perf_buffers[key_id])
+            del self.bpf.perf_buffers[key_id]
             del self._cbs[key]
         else:
             # The key is opened for perf event read
@@ -511,7 +575,7 @@ class PerfEventArray(ArrayBase):
                     exit()
                 else:
                     raise e
-        def lost_cb_(lost):
+        def lost_cb_(_, lost):
             try:
                 lost_cb(lost)
             except IOError as e:
@@ -526,7 +590,7 @@ class PerfEventArray(ArrayBase):
             raise Exception("Could not open perf buffer")
         fd = lib.perf_reader_fd(reader)
         self[self.Key(cpu)] = self.Leaf(fd)
-        self.bpf._add_kprobe((id(self), cpu), reader)
+        self.bpf.perf_buffers[(id(self), cpu)] = reader
         # keep a refcnt
         self._cbs[cpu] = (fn, lost_fn)
         # The actual fd is held by the perf reader, add to track opened keys
@@ -644,6 +708,10 @@ class PerCpuArray(ArrayBase):
     def __setitem__(self, key, leaf):
         super(PerCpuArray, self).__setitem__(key, leaf)
 
+    def __delitem__(self, key):
+        # Delete in this type does not have an effect, so zero out instead
+        self.clearitem(key)
+
     def sum(self, key):
         if isinstance(self.Leaf(), ct.Structure):
             raise IndexError("Leaf must be an integer type for default sum functions")
@@ -665,10 +733,6 @@ class LpmTrie(TableBase):
     def __len__(self):
         raise NotImplementedError
 
-    def __delitem__(self, key):
-        # Not implemented for lpm trie as of kernel commit
-        # b95a5c4db09bc7c253636cb84dc9b12c577fd5a0
-        raise NotImplementedError
 
 class StackTrace(TableBase):
     MAX_DEPTH = 127
@@ -707,10 +771,13 @@ class StackTrace(TableBase):
         for k in self: i += 1
         return i
 
-    def __delitem__(self, key):
-        res = lib.bpf_delete_elem(self.map_fd, ct.byref(key))
-        if res < 0:
-            raise KeyError
-
     def clear(self):
         pass
+
+class DevMap(ArrayBase):
+    def __init__(self, *args, **kwargs):
+        super(DevMap, self).__init__(*args, **kwargs)
+
+class CpuMap(ArrayBase):
+    def __init__(self, *args, **kwargs):
+        super(CpuMap, self).__init__(*args, **kwargs)
