@@ -86,7 +86,7 @@ struct data_t {
 
 BPF_PERF_OUTPUT(events);
 
-int kprobe__sys_mount(struct pt_regs *ctx, char __user *source,
+int syscall__mount(struct pt_regs *ctx, char __user *source,
                       char __user *target, char __user *type,
                       unsigned long flags)
 {
@@ -110,29 +110,29 @@ int kprobe__sys_mount(struct pt_regs *ctx, char __user *source,
     events.perf_submit(ctx, &event, sizeof(event));
 
     event.type = EVENT_MOUNT_SOURCE;
-    memset(event.str, 0, sizeof(event.str));
+    __builtin_memset(event.str, 0, sizeof(event.str));
     bpf_probe_read(event.str, sizeof(event.str), source);
     events.perf_submit(ctx, &event, sizeof(event));
 
     event.type = EVENT_MOUNT_TARGET;
-    memset(event.str, 0, sizeof(event.str));
+    __builtin_memset(event.str, 0, sizeof(event.str));
     bpf_probe_read(event.str, sizeof(event.str), target);
     events.perf_submit(ctx, &event, sizeof(event));
 
     event.type = EVENT_MOUNT_TYPE;
-    memset(event.str, 0, sizeof(event.str));
+    __builtin_memset(event.str, 0, sizeof(event.str));
     bpf_probe_read(event.str, sizeof(event.str), type);
     events.perf_submit(ctx, &event, sizeof(event));
 
     event.type = EVENT_MOUNT_DATA;
-    memset(event.str, 0, sizeof(event.str));
+    __builtin_memset(event.str, 0, sizeof(event.str));
     bpf_probe_read(event.str, sizeof(event.str), data);
     events.perf_submit(ctx, &event, sizeof(event));
 
     return 0;
 }
 
-int kretprobe__sys_mount(struct pt_regs *ctx)
+int do_ret_sys_mount(struct pt_regs *ctx)
 {
     struct data_t event = {};
 
@@ -145,7 +145,7 @@ int kretprobe__sys_mount(struct pt_regs *ctx)
     return 0;
 }
 
-int kprobe__sys_umount(struct pt_regs *ctx, char __user *target, int flags)
+int syscall__umount(struct pt_regs *ctx, char __user *target, int flags)
 {
     struct data_t event = {};
     struct task_struct *task;
@@ -165,14 +165,14 @@ int kprobe__sys_umount(struct pt_regs *ctx, char __user *target, int flags)
     events.perf_submit(ctx, &event, sizeof(event));
 
     event.type = EVENT_UMOUNT_TARGET;
-    memset(event.str, 0, sizeof(event.str));
+    __builtin_memset(event.str, 0, sizeof(event.str));
     bpf_probe_read(event.str, sizeof(event.str), target);
     events.perf_submit(ctx, &event, sizeof(event));
 
     return 0;
 }
 
-int kretprobe__sys_umount(struct pt_regs *ctx)
+int do_ret_sys_umount(struct pt_regs *ctx)
 {
     struct data_t event = {};
 
@@ -382,8 +382,8 @@ def print_event(mounts, umounts, cpu, data, size):
                     flags=decode_umount_flags(syscall['flags']),
                     retval=decode_errno(event.union.retval))
             print('{:16} {:<7} {:<7} {:<11} {}'.format(
-                syscall['comm'].decode(), syscall['tgid'], syscall['pid'],
-                syscall['mnt_ns'], call))
+                syscall['comm'].decode('utf-8', 'replace'), syscall['tgid'],
+                syscall['pid'], syscall['mnt_ns'], call))
     except KeyError:
         # This might happen if we lost an event.
         pass
@@ -393,17 +393,32 @@ def main():
     parser = argparse.ArgumentParser(
         description='trace mount() and umount() syscalls'
     )
+    parser.add_argument("--ebpf", action="store_true",
+        help=argparse.SUPPRESS)
     args = parser.parse_args()
 
     mounts = {}
     umounts = {}
+    if args.ebpf:
+        print(bpf_text)
+        exit()
     b = bcc.BPF(text=bpf_text)
+    mount_fnname = b.get_syscall_fnname("mount")
+    b.attach_kprobe(event=mount_fnname, fn_name="syscall__mount")
+    b.attach_kretprobe(event=mount_fnname, fn_name="do_ret_sys_mount")
+    umount_fnname = b.get_syscall_fnname("umount")
+    b.attach_kprobe(event=umount_fnname, fn_name="syscall__umount")
+    b.attach_kretprobe(event=umount_fnname, fn_name="do_ret_sys_umount")
     b['events'].open_perf_buffer(
         functools.partial(print_event, mounts, umounts))
     print('{:16} {:<7} {:<7} {:<11} {}'.format(
         'COMM', 'PID', 'TID', 'MNT_NS', 'CALL'))
     while True:
-        b.kprobe_poll()
+        try:
+            b.perf_buffer_poll()
+        except KeyboardInterrupt:
+            exit()
+
 
 
 if __name__ == '__main__':
