@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 #
 # bashreadline  Print entered bash commands from all running shells.
 #               For Linux, uses BCC, eBPF. Embedded C.
@@ -22,7 +22,7 @@ int printret(struct pt_regs *ctx) {
         return 0;
 
     char str[80] = {};
-    bpf_probe_read(&str, sizeof(str), (void *)ctx->ax);
+    bpf_probe_read(&str, sizeof(str), (void *)PT_REGS_RC(ctx));
     bpf_trace_printk("%s\\n", &str);
 
     return 0;
