@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 # @lint-avoid-python-3-compatibility-imports
 #
 # profile  Profile CPU usage by sampling stack traces at a timed interval.
@@ -102,6 +102,8 @@ parser.add_argument("--stack-storage-size", default=16384,
 parser.add_argument("duration", nargs="?", default=99999999,
     type=positive_nonzero_int,
     help="duration of trace, in seconds")
+parser.add_argument("-C", "--cpu", type=int, default=-1,
+    help="cpu number to run profile on")
 parser.add_argument("--ebpf", action="store_true",
     help=argparse.SUPPRESS)
 
@@ -143,7 +145,6 @@ int do_perf_event(struct bpf_perf_event_data *ctx) {
         return 0;
 
     // create map key
-    u64 zero = 0, *val;
     struct key_t key = {.pid = pid};
     bpf_get_current_comm(&key.name, sizeof(key.name));
 
@@ -153,9 +154,7 @@ int do_perf_event(struct bpf_perf_event_data *ctx) {
 
     if (key.kernel_stack_id >= 0) {
         // populate extras to fix the kernel stack
-        struct pt_regs regs = {};
-        bpf_probe_read(&regs, sizeof(regs), (void *)&ctx->regs);
-        u64 ip = PT_REGS_IP(&regs);
+        u64 ip = PT_REGS_IP(&ctx->regs);
         u64 page_offset;
 
         // if ip isn't sane, leave key ips as zero for later checking
@@ -180,8 +179,7 @@ int do_perf_event(struct bpf_perf_event_data *ctx) {
         }
     }
 
-    val = counts.lookup_or_init(&key, &zero);
-    (*val)++;
+    counts.increment(key);
     return 0;
 }
 """
@@ -232,6 +230,8 @@ sample_context = "%s%d %s" % (("", sample_freq, "Hertz") if sample_freq
 if not args.folded:
     print("Sampling at %s of %s by %s stack" %
         (sample_context, thread_context, stack_context), end="")
+    if args.cpu >= 0:
+        print(" on CPU#{}".format(args.cpu), end="")
     if duration < 99999999:
         print(" for %d secs." % duration)
     else:
@@ -246,7 +246,7 @@ if debug or args.ebpf:
 b = BPF(text=bpf_text)
 b.attach_perf_event(ev_type=PerfType.SOFTWARE,
     ev_config=PerfSWConfig.CPU_CLOCK, fn_name="do_perf_event",
-    sample_period=sample_period, sample_freq=sample_freq)
+    sample_period=sample_period, sample_freq=sample_freq, cpu=args.cpu)
 
 # signal handler
 def signal_ignore(signal, frame):
@@ -268,7 +268,7 @@ if not args.folded:
 
 def aksym(addr):
     if args.annotations:
-        return b.ksym(addr) + "_[k]"
+        return b.ksym(addr) + "_[k]".encode()
     else:
         return b.ksym(addr)
 
@@ -306,7 +306,7 @@ for k, v in sorted(counts.items(), key=lambda counts: counts[1].value):
         # print folded stack output
         user_stack = list(user_stack)
         kernel_stack = list(kernel_stack)
-        line = [k.name.decode()]
+        line = [k.name]
         # if we failed to get the stack is, such as due to no space (-ENOMEM) or
         # hash collision (-EEXIST), we still print a placeholder for consistency
         if not args.kernel_stacks_only:
@@ -320,7 +320,7 @@ for k, v in sorted(counts.items(), key=lambda counts: counts[1].value):
                 line.append("[Missed Kernel Stack]")
             else:
                 line.extend([b.ksym(addr) for addr in reversed(kernel_stack)])
-        print("%s %d" % (";".join(line), v.value))
+        print("%s %d" % (b";".join(line).decode('utf-8', 'replace'), v.value))
     else:
         # print default multi-line stack output
         if not args.user_stacks_only:
@@ -336,8 +336,8 @@ for k, v in sorted(counts.items(), key=lambda counts: counts[1].value):
                 print("    [Missed User Stack]")
             else:
                 for addr in user_stack:
-                    print("    %s" % b.sym(addr, k.pid))
-        print("    %-16s %s (%d)" % ("-", k.name.decode(), k.pid))
+                    print("    %s" % b.sym(addr, k.pid).decode('utf-8', 'replace'))
+        print("    %-16s %s (%d)" % ("-", k.name.decode('utf-8', 'replace'), k.pid))
         print("        %d\n" % v.value)
 
 # check missing
